@@ -11,7 +11,7 @@ class AIRClientConfig:
     schedule_asset_id: int
     schedule_asset_ext_id: str
     data_fields: dict
-    data_fields_defs: list
+    data_fields_defs: dict
     model_name: str
     model_version: str
 
