@@ -1,4 +1,4 @@
-from typing import Any, Dict, List
+from typing import Any, Dict, List, Optional
 
 from cognite.air._api import AIRClientError
 from cognite.air._backfilling_api import AIRBackfillingAPI, NoBackfillAPI
@@ -28,7 +28,7 @@ class AIRClient:
             schedule_asset_id=schedule_asset.id,
             schedule_asset_ext_id=sa_ext_id,
             data_fields=parse_json_if_json(schedule_asset.metadata.get(SA_FIELD_META_DATA)) or {},
-            data_fields_defs=parse_json_if_json(model_asset.metadata.get(MA_FIELD_META_FIELDS)) or {},
+            data_fields_defs=self.create_data_field_dct(model_asset.metadata.get(MA_FIELD_META_FIELDS)),
             model_name=model_asset.name,
             model_version=model_version,
         )
@@ -76,16 +76,40 @@ class AIRClient:
 
         fields = list(map(self._config.data_fields.get, field_names))
         if not ignore_unknown_field_names and None in fields:
-            err_field_names = [name for name, field in zip(field_names, fields) if field is None]
+            err_field_names = [id for id, field in zip(field_names, fields) if field is None]
             raise ValueError(f"The following field names were not found: {err_field_names}")
 
-        return list(map(parse_json_if_json, fields))  # type: ignore
+        return_fields = []
+        for id, field in zip(field_names, fields):
+            field_type = self._config.data_fields_defs[id]["type"]
+            ret_val = parse_json_if_json(field)
+            if isinstance(ret_val, list):  # this means multiple=True
+                return_fields.append(list(map(field_type, ret_val)))
+            else:
+                return_fields.append(field_type(ret_val))
+        return return_fields  # type: ignore
 
     def retrieve_field(self, field_name: str) -> str:
         if not isinstance(field_name, str):
             raise TypeError(f"Expected 'field_name' to be of type {str}, not {type(field_name)}")
         return self.retrieve_fields([field_name])[0]
 
+    @staticmethod
+    def convert_type_info(dct):
+        legal_types = {
+            "str": str,
+            "int": int,
+            "float": float,
+            "TimeSeries": str,
+            "Asset": str,
+        }
+        dct["type"] = legal_types[dct["type"]]
+        return dct
+
+    def create_data_field_dct(self, json_str: Optional[str]):
+        lst_of_field_defs = parse_json_if_json(json_str) or []
+        return {dct.pop("id"): self.convert_type_info(dct) for dct in lst_of_field_defs}
+
     @staticmethod
     def _extract_and_validate_sa_ext_id(data):
         sa_ext_id = data.get(SA_EXT_ID)
